=== modified file 'NEWS'
--- NEWS	2009-05-11 13:46:33 +0000
+++ NEWS	2009-05-21 20:26:23 +0000
@@ -16,6 +16,11 @@
 
   * Mark as compatible with bzr 1.13.
 
+ FEATURES
+
+  * Support storing credentials in the GNOME Keyring if it is 
+    available. (Jelmer Vernooij)
+
 0.95.0	2008-08-04
 
  UI

=== modified file '__init__.py'
--- __init__.py	2009-05-07 09:41:56 +0000
+++ __init__.py	2009-05-21 20:26:23 +0000
@@ -36,7 +36,10 @@
 
 import bzrlib
 import bzrlib.api
-from bzrlib import errors
+from bzrlib import (
+    config,
+    errors,
+    )
 from bzrlib.commands import plugin_cmds
 
 import os.path
@@ -150,6 +153,19 @@
         return "No DISPLAY. Unable to run GTK+ application."
 
 
+credential_store_registry = getattr(config, "credential_store_registry", None)
+if credential_store_registry is not None:
+    try:
+        credential_store_registry.register_lazy(
+            "gnome-keyring", "bzrlib.plugins.gtk.keyring", "GnomeKeyringCredentialStore",
+            help="The GNOME Keyring.", fallback=True)
+    except TypeError:
+    # Fallback credentials stores were introduced in Bazaar 1.15
+        credential_store_registry.register_lazy(
+            "gnome-keyring", "bzrlib.plugins.gtk.keyring", "GnomeKeyringCredentialStore",
+            help="The GNOME Keyring.")
+
+
 def load_tests(basic_tests, module, loader):
     testmod_names = [
         'tests',

=== added file 'keyring.py'
--- keyring.py	1970-01-01 00:00:00 +0000
+++ keyring.py	2009-05-21 20:26:23 +0000
@@ -0,0 +1,78 @@
+# Copyright (C) 2009 Jelmer Vernooij <jelmer@samba.org>
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import gobject
+try:
+    import gnomekeyring
+except ImportError:
+    gnomekeyring = None
+
+from bzrlib.config import (
+    CredentialStore,
+    )
+
+
+class GnomeKeyringCredentialStore(CredentialStore):
+
+    def __init__(self):
+        CredentialStore.__init__(self)
+        # External apps that load bzrlib may also set this, so 
+        # don't override:
+        if gobject.get_application_name() is None:
+            gobject.set_application_name("bzr")
+
+    def decode_password(self, credentials):
+        if gnomekeyring is None:
+            return None
+        try:
+            attrs = {}
+            if "scheme" in credentials:
+                attrs["protocol"] = credentials["scheme"]
+            if "host" in credentials:
+                attrs["server"] = credentials["host"]
+            if "user" in credentials:
+                attrs["user"] = credentials["user"]
+            if "port" in credentials:
+                attrs["port"] = str(credentials["port"])
+            items = gnomekeyring.find_items_sync(gnomekeyring.ITEM_GENERIC_SECRET, 
+                                                 attrs)
+            return items[0].secret
+        except (gnomekeyring.NoMatchError, gnomekeyring.DeniedError):
+            return None
+
+    def get_credentials(self, scheme, host, port=None, user=None, path=None, 
+                        realm=None):
+        if gnomekeyring is None:
+            return None
+        attrs = {
+            "protocol": scheme,
+            "server": host,
+            }
+        if realm is not None:
+            attrs["realm"] = realm
+        if port is not None:
+            attrs["port"] = str(port)
+        if user is not None:
+            attrs["user"] = user
+        credentials = { "scheme": scheme, "host": host, "port": port, 
+            "realm": realm, "user": user}
+        try:
+            items = gnomekeyring.find_items_sync(
+                gnomekeyring.ITEM_NETWORK_PASSWORD, attrs)
+            credentials["user"] = items[0].attributes["user"]
+            credentials["password"] = items[0].secret
+        except (gnomekeyring.NoMatchError, gnomekeyring.DeniedError):
+            return None

